"""OTElib

A package to interact with the Open Translation Environment (OTE) API.
"""
from .client import OTEClient

__all__ = ("OTEClient",)

__version__ = "0.3.2"
__author__ = "SINTEF"
__author_email__ = "Team4.0@SINTEF.no"
